@@ -315,6 +315,20 @@ class JaxBackend(jax_backend.JaxBackend):  # type: ignore
             return a.astype(getattr(jnp, dtype))
         return a.astype(dtype)
 
+    def arange(self, start: int, stop: Optional[int] = None, step: int = 1) -> Tensor:
+        if stop is None:
+            return jnp.arange(start=0, stop=start, step=step)
+        return jnp.arange(start=start, stop=stop, step=step)
+
+    def mod(self, x: Tensor, y: Tensor) -> Tensor:
+        return jnp.mod(x, y)
+
+    def right_shift(self, x: Tensor, y: Tensor) -> Tensor:
+        return jnp.right_shift(x, y)
+
+    def left_shift(self, x: Tensor, y: Tensor) -> Tensor:
+        return jnp.left_shift(x, y)
+
     def expm(self, a: Tensor) -> Tensor:
         return jsp.linalg.expm(a)
         # currently expm in jax doesn't support AD, it will raise an AssertError,
@@ -378,6 +392,15 @@ class JaxBackend(jax_backend.JaxBackend):  # type: ignore
     def solve(self, A: Tensor, b: Tensor, assume_a: str = "gen") -> Tensor:
         return jsp.linalg.solve(A, b, assume_a)
 
+    def tree_map(self, f: Callable[..., Any], *pytrees: Any) -> Any:
+        return libjax.tree_map(f, *pytrees)
+
+    def tree_flatten(self: Any, pytree: Any) -> Tuple[Any, Any]:
+        return libjax.tree_flatten(pytree)  # type: ignore
+
+    def tree_unflatten(self: Any, treedef: Any, leaves: Any) -> Any:
+        return libjax.tree_unflatten(treedef, leaves)
+
     def set_random_state(
         self, seed: Optional[Union[int, PRNGKeyArray]] = None, get_only: bool = False
     ) -> Any:
