@@ -23,6 +23,7 @@ except ImportError:
 
 dtypestr: str
 Tensor = Any
+pytree = Any
 
 torchlib: Any
 
@@ -34,6 +35,28 @@ logger = logging.getLogger(__name__)
 # To be added once pytorch backend is ready
 
 
+class torch_optimizer:
+    def __init__(self, optimizer: Any) -> None:
+        self.optimizer = optimizer
+        self.is_init = False
+
+    def update(self, grads: pytree, params: pytree) -> pytree:
+        # flatten grad and param
+        params, treedef = PyTorchBackend.tree_flatten(None, params)
+        grads, _ = PyTorchBackend.tree_flatten(None, grads)
+        if self.is_init is False:
+            self.optimizer = self.optimizer(params)
+            self.is_init = True
+        with torchlib.no_grad():
+            for g, p in zip(grads, params):
+                p.grad = g
+        self.optimizer.step()
+        self.optimizer.zero_grad()
+        # reorg the param
+        params = PyTorchBackend.tree_unflatten(None, treedef, params)
+        return params
+
+
 def _conj_torch(self: Any, tensor: Tensor) -> Tensor:
     t = torchlib.conj(tensor)
     return t.resolve_conj()  # any side effect?
@@ -352,9 +375,36 @@ class PyTorchBackend(pytorch_backend.PyTorchBackend):  # type: ignore
             return a.type(getattr(torchlib, dtype))
         return a.type(dtype)
 
+    def arange(self, start: int, stop: Optional[int] = None, step: int = 1) -> Tensor:
+        if stop is None:
+            return torchlib.arange(start=0, end=start, step=step)
+        return torchlib.arange(start=start, end=stop, step=step)
+
+    def mod(self, x: Tensor, y: Tensor) -> Tensor:
+        return torchlib.fmod(x, y)
+
+    def right_shift(self, x: Tensor, y: Tensor) -> Tensor:
+        return torchlib.bitwise_right_shift(x, y)
+
+    def left_shift(self, x: Tensor, y: Tensor) -> Tensor:
+        return torchlib.bitwise_left_shift(x, y)
+
     def solve(self, A: Tensor, b: Tensor, **kws: Any) -> Tensor:
         return torchlib.linalg.solve(A, b)
 
+    def reverse(self, a: Tensor) -> Tensor:
+        return torchlib.flip(a, dims=(-1,))
+
+    def tree_map(self, f: Callable[..., Any], *pytrees: Any) -> Any:
+        # TODO(@refraction-ray): torch not support multiple pytree args
+        return torchlib.utils._pytree.tree_map(f, *pytrees)
+
+    def tree_flatten(self: Any, pytree: Any) -> Tuple[Any, Any]:
+        return torchlib.utils._pytree.tree_flatten(pytree)  # type: ignore
+
+    def tree_unflatten(self: Any, treedef: Any, leaves: Any) -> Any:
+        return torchlib.utils._pytree.tree_unflatten(leaves, treedef)
+
     def cond(
         self,
         pred: bool,
@@ -413,6 +463,13 @@ class PyTorchBackend(pytorch_backend.PyTorchBackend):  # type: ignore
         argnums: Union[int, Sequence[int]] = 0,
         has_aux: bool = False,
     ) -> Callable[..., Tuple[Any, Any]]:
+        def ask_require(t: Tensor) -> Any:
+            t.requires_grad_(True)
+            return t
+
+        def get_grad(t: Tensor) -> Tensor:
+            return t.grad
+
         def wrapper(*args: Any, **kws: Any) -> Any:
             x = []
             if isinstance(argnums, int):
@@ -423,7 +480,7 @@ class PyTorchBackend(pytorch_backend.PyTorchBackend):  # type: ignore
                 argnumsl = argnums  # type: ignore
             for i, arg in enumerate(args):
                 if i in argnumsl:
-                    x.append(arg.requires_grad_(True))
+                    x.append(self.tree_map(ask_require, arg))
                 else:
                     x.append(arg)
             y = f(*x, **kws)
@@ -431,7 +488,7 @@ class PyTorchBackend(pytorch_backend.PyTorchBackend):  # type: ignore
                 y[0].backward()
             else:
                 y.backward()
-            gs = [x[i].grad for i in argnumsl]
+            gs = [self.tree_map(get_grad, x[i]) for i in argnumsl]
             if len(gs) == 1:
                 gs = gs[0]
             return y, gs
@@ -532,3 +589,5 @@ class PyTorchBackend(pytorch_backend.PyTorchBackend):  # type: ignore
         return f
 
     vvag = vectorized_value_and_grad
+
+    optimizer = torch_optimizer
