@@ -198,8 +198,9 @@ def _rq_tf(
         phases = tf.math.sign(tf.linalg.diag_part(r))
         q = q * phases
         r = phases[:, None] * r
-    r, q = tf.math.conj(tf.transpose(r)), tf.math.conj(
-        tf.transpose(q)
+    r, q = (
+        tf.math.conj(tf.transpose(r)),
+        tf.math.conj(tf.transpose(q)),
     )  # M=r*q at this point
     center_dim = tf.shape(r)[1]
     r = tf.reshape(r, tf.concat([left_dims, [center_dim]], axis=-1))
@@ -413,6 +414,20 @@ class TensorFlowBackend(tensorflow_backend.TensorFlowBackend):  # type: ignore
             return tf.cast(a, dtype=getattr(tf, dtype))
         return tf.cast(a, dtype=dtype)
 
+    def arange(self, start: int, stop: Optional[int] = None, step: int = 1) -> Tensor:
+        if stop is None:
+            return tf.range(start=0, limit=start, delta=step)
+        return tf.range(start=start, limit=stop, delta=step)
+
+    def mod(self, x: Tensor, y: Tensor) -> Tensor:
+        return tf.math.mod(x, y)
+
+    def right_shift(self, x: Tensor, y: Tensor) -> Tensor:
+        return tf.bitwise.right_shift(x, y)
+
+    def left_shift(self, x: Tensor, y: Tensor) -> Tensor:
+        return tf.bitwise.left_shift(x, y)
+
     def solve(self, A: Tensor, b: Tensor, **kws: Any) -> Tensor:
         if b.shape[-1] == A.shape[-1]:
             b = b[..., tf.newaxis]
