@@ -16,6 +16,7 @@ import tensornetwork as tn
 from tensornetwork.backend_contextmanager import get_default_backend
 
 from .backends import get_backend  # type: ignore
+from .simplify import _multi_remove
 
 logger = logging.getLogger(__name__)
 
@@ -31,6 +32,7 @@ modules = [
     "tensorcircuit.densitymatrix2",
     "tensorcircuit.channels",
     "tensorcircuit.keras",
+    "tensorcircuit.torchnn",
     "tensorcircuit.quantum",
     "tensorcircuit.simplify",
     "tensorcircuit.interfaces",
@@ -239,11 +241,6 @@ def runtime_dtype(dtype: Optional[str] = None) -> Iterator[Tuple[str, str]]:
 # but correctness has not been extensively tested for some of them)
 
 
-def _multi_remove(elems: List[Any], indices: List[int]) -> List[Any]:
-    """Remove multiple indices from a list for one time."""
-    return [i for j, i in enumerate(elems) if j not in indices]
-
-
 def _sizen(node: tn.Node, is_log: bool = False) -> int:
     s = reduce(mul, node.tensor.shape + (1,))
     if is_log:
@@ -261,9 +258,17 @@ def _merge_single_gates(
     queue = [n for n in nodes if len(n.tensor.shape) <= 2]
     while queue:
         n0 = queue[0]
+        try:
+            n0[0]
+        except IndexError:
+            queue = _multi_remove(queue, [0])
+            continue
         if n0[0].is_dangling():
             try:
                 e0 = n0[1]
+                if e0.is_dangling():
+                    queue = _multi_remove(queue, [0])
+                    continue
             except IndexError:
                 queue = _multi_remove(queue, [0])
                 continue
@@ -271,7 +276,6 @@ def _merge_single_gates(
             e0 = n0[0]
         njs = [i for i, n in enumerate(nodes) if id(n) in [id(e0.node1), id(e0.node2)]]
         qjs = [i for i, n in enumerate(queue) if id(n) in [id(e0.node1), id(e0.node2)]]
-
         new_node = tn.contract(e0)
         total_size += _sizen(new_node)  # type: ignore
 
@@ -279,8 +283,11 @@ def _merge_single_gates(
             _sizen(new_node, is_log=True),
         )
         queue = _multi_remove(queue, qjs)
-        nodes[njs[1]] = new_node
-        nodes = _multi_remove(nodes, [njs[0]])
+        if len(njs) > 1:
+            nodes[njs[1]] = new_node
+            nodes = _multi_remove(nodes, [njs[0]])
+        else:  # trace edge?
+            nodes[njs[0]] = new_node
         if len(new_node.tensor.shape) <= 2:
             # queue.append(new_node)
             queue.insert(0, new_node)
@@ -654,6 +661,7 @@ def custom(
 
     total_size = None
     if kws.get("preprocessing", None):
+        # nodes = _full_light_cone_cancel(nodes)
         nodes, total_size = _merge_single_gates(nodes)
     if not isinstance(optimizer, list):
         alg = partial(optimizer, memory_limit=memory_limit)
@@ -807,7 +815,7 @@ def set_contractor(
     return cf
 
 
-set_contractor()
+set_contractor("greedy", preprocessing=True)
 
 get_contractor = partial(set_contractor, set_global=False)
 
