@@ -245,6 +245,35 @@ def test_backend_methods_2(backend):
     np.testing.assert_allclose(
         tc.backend.eigvalsh(tc.backend.ones([2, 2])), np.array([0, 2]), atol=1e-5
     )
+    np.testing.assert_allclose(
+        tc.backend.left_shift(
+            tc.backend.convert_to_tensor(np.array([4, 3])),
+            tc.backend.convert_to_tensor(np.array([1, 1])),
+        ),
+        np.array([8, 6]),
+    )
+    np.testing.assert_allclose(
+        tc.backend.right_shift(
+            tc.backend.convert_to_tensor(np.array([4, 3])),
+            tc.backend.convert_to_tensor(np.array([1, 1])),
+        ),
+        np.array([2, 1]),
+    )
+    np.testing.assert_allclose(
+        tc.backend.mod(
+            tc.backend.convert_to_tensor(np.array([4, 3])),
+            tc.backend.convert_to_tensor(np.array([2, 2])),
+        ),
+        np.array([0, 1]),
+    )
+    np.testing.assert_allclose(
+        tc.backend.arange(3),
+        np.array([0, 1, 2]),
+    )
+    np.testing.assert_allclose(
+        tc.backend.arange(1, 5, 2),
+        np.array([1, 3]),
+    )
 
 
 @pytest.mark.parametrize("backend", [lf("npb"), lf("tfb"), lf("jaxb"), lf("torchb")])
@@ -779,7 +808,16 @@ def test_solve(backend):
     np.testing.assert_allclose(xp, x[:, 0], atol=1e-5)
 
 
-@pytest.mark.parametrize("backend", [lf("tfb"), lf("jaxb")])
+@pytest.mark.parametrize("backend", [lf("npb"), lf("tfb"), lf("jaxb"), lf("torchb")])
+def test_treeutils(backend):
+    d0 = {"a": np.ones([2]), "b": [tc.backend.zeros([]), tc.backend.ones([1, 1])]}
+    leaves, treedef = tc.backend.tree_flatten(d0)
+    d1 = tc.backend.tree_unflatten(treedef, leaves)
+    d2 = tc.backend.tree_map(lambda x: 2 * x, d1)
+    np.testing.assert_allclose(2 * np.ones([1, 1]), d2["b"][1])
+
+
+@pytest.mark.parametrize("backend", [lf("tfb"), lf("jaxb"), lf("torchb")])
 def test_optimizers(backend):
     if tc.backend.name == "jax":
         try:
@@ -787,6 +825,12 @@ def test_optimizers(backend):
         except ImportError:
             pytest.skip("optax is not installed")
 
+    if tc.backend.name == "pytorch":
+        try:
+            import torch
+        except ImportError:
+            pytest.skip("torch is not installed")
+
     def f(params, n):
         c = tc.Circuit(n)
         c = tc.templates.blocks.example_block(c, params["a"])
@@ -802,6 +846,9 @@ def test_optimizers(backend):
         elif tc.backend.name == "jax":
             optimizer2 = optax.adam(5e-2)
             opt = tc.backend.optimizer(optimizer2)
+        elif tc.backend.name == "pytorch":
+            optimizer3 = partial(torch.optim.Adam, lr=5e-2)
+            opt = tc.backend.optimizer(optimizer3)
         else:
             raise ValueError("%s doesn't support optimizer interface" % tc.backend.name)
         return opt
@@ -810,8 +857,8 @@ def test_optimizers(backend):
     opt = get_opt()
 
     params = {
-        "a": tc.backend.implicit_randn([4, n]),
-        "b": tc.backend.implicit_randn([4, n]),
+        "a": tc.backend.ones([4, n], dtype="float32"),
+        "b": tc.backend.ones([4, n], dtype="float32"),
     }
 
     for _ in range(20):
@@ -828,12 +875,11 @@ def test_optimizers(backend):
 
     vgs2 = tc.backend.jit(tc.backend.value_and_grad(f2, argnums=0), static_argnums=1)
 
-    params = tc.backend.implicit_randn([4, n])
+    params = tc.backend.ones([4, n], dtype="float32")
     opt = get_opt()
 
     for _ in range(20):
         loss, grads = vgs2(params, n)
-        print(grads, params)
         params = opt.update(grads, params)
         print(loss)
 
